# Copyright (C) 2020-2023 MicroSys Electronics GmbH

inherit microsys

FILESEXTRAPATHS:prepend := "${THISDIR}/${PN}:"
PATCHTOOL = "git"
PATCH_COMMIT_FUNCTIONS = "1"

#URL    = "git://github.com/MicroSysElectronics/u-boot;protocol=https"
#BRANCH = "microsys-s32-bsp34.0"
#SRCREV = "9c4b68131c22338ebb31a7de91930d3f8ce01a8d"

BRANCH = "release/bsp36.0-2020.04"
SRCREV = "ec57ab5ab89c86df5749a67a3ef23e7fe28c87a5"
SRC_URI:append = " file://u-boot.patch"

# devtool wants to copy .config from here...
#B = "${WORKDIR}/${BPN}-${PV}"
DEVTOOL_DISABLE_MENUCONFIG = "1"

DEPENDS += "python-native"

# u-boot-nxp-hse.inc already adds this (in layer meta-alb):
#DEPENDS:append = "${@bb.utils.contains('DISTRO_FEATURES', 'hse', ' hse-firmware ', '', d)}"

# Take care that HSE firmware is installed
# before do_patch() is executed:
do_patch[depends] += "${@bb.utils.contains('DISTRO_FEATURES', 'hse', "hse-firmware:do_install", '', d)}"
do_patch[deptask] += "${@bb.utils.contains('DISTRO_FEATURES', 'hse', "hse-firmware:do_install", '', d)}"

python do_patch:prepend() {
    if d.getVar('PATCHTOOL') == 'git':
        srcsubdir = d.getVar('S')
        bb.process.run('git config am.threeWay true', cwd=srcsubdir)
        bb.process.run('git config am.keepcr true', cwd=srcsubdir)
        bb.process.run('git config apply.whitespace fix', cwd=srcsubdir)
        bb.process.run('git config apply.ignoreWhitespace change', cwd=srcsubdir)
}

python do_patch:append() {

    import os

    distro_features = d.getVar('DISTRO_FEATURES').split()

    if 'hse' in distro_features:
        src = d.getVar('S')
        uboot_machine = d.getVar('UBOOT_MACHINE').split()

        hse_fw = os.path.join(microsys_get_hse_fw_dir(d), microsys_get_hse_fw_image(d))
        if not os.path.isfile(hse_fw):
            bb.warn("%s: no such file!" % hse_fw)
            bb.fatal("HSE firmware not found!")

        bb.plain("HSE firmware: %s" % hse_fw)
        d.setVar("HSE_FW", hse_fw)

        # Patch the configuration files here in do_patch().
        # This saves calling do_add_hse_fw_path() and
        # do_merge_delta_config() later:
        for config in uboot_machine:
            cfg_file = os.path.join(src, 'configs', config)
            if not os.path.isfile(cfg_file):
                bb.fatal("%s: no such file!" % cfg_file)
            bb.note("Patching %s" % cfg_file)
            bb.note("HSW_FW = %s" % hse_fw)
            with open(cfg_file, "a") as cfg:
                print("CONFIG_NXP_HSE_SUPPORT=y", file=cfg)
                print('CONFIG_NXP_HSE_FW_FILE="'+hse_fw+'"', file=cfg)
        bb.process.run("git commit --all --message='do_patch:append(): HSE configuration' --signoff --no-edit", cwd=src)
}

# Don't execute these two tasks, because
# the job is done in do_patch::append() above:
do_add_hse_fw_path[noexec] = "1"
do_merge_delta_config[noexec] = "1"
